/**************************************************************************//**
 * @file
 * @brief LCD test/demo routines
 * @author Energy Micro AS
 * @version 1.1.1
 ******************************************************************************
 * @section License
 * <b>(C) Copyright 2009 Energy Micro AS, http://www.energymicro.com</b>
 ******************************************************************************
 *
 * This source code is the property of Energy Micro AS. The source and compiled
 * code may only be used on Energy Micro "EFM32" microcontrollers.
 *
 * This copyright notice may not be removed from the source code nor changed.
 *
 * DISCLAIMER OF WARRANTY/LIMITATION OF REMEDIES: Energy Micro AS has no
 * obligation to support this Software. Energy Micro AS is providing the
 * Software "AS IS", with no express or implied warranties of any kind,
 * including, but not limited to, any implied warranties of merchantability
 * or fitness for any particular purpose or warranties against infringement
 * of any proprietary rights of a third party.
 *
 * Energy Micro AS will not be liable for any consequential, incidental, or
 * special damages, or any other relief, or for any claim by any third party,
 * arising from your use of this Software.
 *
 *****************************************************************************/
#include <stdio.h>
#include <stdbool.h>
#include <string.h>
#include <stdlib.h>
#include "efm32.h"
#include "lcdcontroller.h"
#include "vcmp.h"

/** SysTick polled Delay routine */
extern void Delay(uint32_t dlyTicks);

void ArrowsCheck(LCD_TypeDef *lcd);
void BatteryCheck(LCD_TypeDef *lcd);
void KillDevice(void);

/** Demo scroll text */
static char *stext = "OLIMEX Ltd. http://www.olimex.com        ";
static bool bzEnable = false;

/**************************************************************************//**
 * @brief LCD scrolls a text over the display, sort of "polled printf"
 * @param lcd Pointer to LCD register block
 *****************************************************************************/
bool LCD_ScrollText(LCD_TypeDef *lcd, char *scrolltext, bool start)
{
	int len;
	static int i;
	char buffer[8];
	
	buffer[7] = 0x00;
	len = strlen(scrolltext);
	if (len < 7)
		return false;
	
	if(start)
		i = 0;
	
	if(i < (len - 7))
	{
		memcpy(buffer, scrolltext + i, 7);
		LCD_Write(lcd, buffer);
		i++;
		return true;
	}
	else
		return false;
}


/**************************************************************************//**
 * @brief LCD Blink Test
 * @param lcd Pointer to LCD register block
 *****************************************************************************/
void LCD_BlinkTest(LCD_TypeDef *lcd)
{
//  LCD_EnergyMode(lcd, 0, 1);
//  LCD_EnergyMode(lcd, 1, 1);
//  LCD_EnergyMode(lcd, 2, 1);
//  LCD_EnergyMode(lcd, 3, 1);
//  LCD_EnergyMode(lcd, 4, 1);
//  LCD_Number(lcd, 2359);
//  LCD_Symbol(lcd, LCD_SYMBOL_COL10, 1);
//  LCD_Symbol(lcd, LCD_SYMBOL_GECKO, 1);
//  LCD_Symbol(lcd, LCD_SYMBOL_EFM32, 1);
//  LCD_Write(lcd, " EFM32 ");
//  lcd->BACTRL |= LCD_BACTRL_BLINKEN;
//  while (lcd->SYNCBUSY) ;
//  Delay(2000);
//  LCD_EnergyMode(lcd, 4, 0);
//  Delay(62);
//  LCD_EnergyMode(lcd, 3, 0);
//  Delay(62);
//  LCD_EnergyMode(lcd, 2, 0);
//  Delay(62);
//  LCD_EnergyMode(lcd, 1, 0);
//  Delay(62);
//  LCD_EnergyMode(lcd, 0, 0);
//  lcd->BACTRL &= ~LCD_BACTRL_BLINKEN;
//  while (lcd->SYNCBUSY) ;
}


bool LCD_TextMagic(LCD_TypeDef *lcd, bool start)
{
	static char animation[][7] = {
		"OOOOOOO",
		"XXXXXXX",
		"+++++++",
		"@@@@@@@",
		"OLIMEX ",
		"@@IMEX ",
		"P@@MEX ",
		"PL@@EX ",
		"PLO@@X ",
		"PLOV@@ ",
		"PLOVD@@",
		"PLOVDI@",
		"PLOVDIV",
	};
			
	
	static int counter = 0;
	
	if(start)
		counter = 0;

	if(counter < (sizeof(animation) / sizeof(animation[0])) ) {
		LCD_Write(lcd, animation[counter++]);
		return true;
	}
	else {
		return false;
	}
}

/**************************************************************************//**
 * @brief LCD Test Routine, shows various text and patterns
 *****************************************************************************/
void LCD_Test(LCD_TypeDef *lcd)
{
	int i, numberOfIterations = 1000;

	LCD_Symbol(lcd, LCD_SYMBOL_OLIMEX, 1);
	LCD_Battery(lcd, 1);

	/* Exercise for the reader: Go to EM2 here and on all delays... */
	bool testModeOn = true;
	int testIndex = 5, waitTime = 0;
	while(testModeOn) {
		switch(testIndex) {
		case 0: /* NUMBER COUNTDOWN */
			LCD_AllOff(lcd);
			i = 100;
			LCD_Number(lcd, i);
			waitTime = 10;
			testIndex++;
			break;
			
		case 1:
			if(--i) {
				LCD_Number(lcd, i);
				waitTime = 10;
			}
			else {
				LCD_NumberOff(lcd);
				waitTime = 0;
				testIndex++;
			}
			break;
			
		case 2: /* OLIMEX LOGO */
			LCD_Write(lcd, "OLIMEX ");
			waitTime = 2000;
			testIndex++;
			break;

		case 3: /* TOGGLE ALL ON/OFF */
			LCD_AllOn(lcd);
			waitTime = 1000;
			testIndex++;
			break;

		case 4:
			LCD_AllOff(lcd);
			waitTime = 1000;
			testIndex++;
			break;

		case 5: /* TEXT MAGIC */
			LCD_TextMagic(lcd, true);
			waitTime = 80;
			testIndex++;
			break;
			
		case 6:
			if( !LCD_TextMagic(lcd, false) ) {
//				testIndex++;
				testIndex = 9;
			}
			waitTime = 80;
			break;
		
		case 7: /* BATTERY EYE CANDY */
			i = 1;
			LCD_Battery(lcd, i);
			waitTime = 200;
			testIndex++;
			break;
			
		case 8:
			if(i < 5) {
				LCD_Battery(lcd, i++);
				waitTime = 1000;
			}
			else {
				waitTime = 0;
				testIndex++;
			}
			break;
			
		case 9: /* SCROLLING TEXT */
			LCD_ScrollText(lcd, stext, true);
			waitTime = 1000;
			testIndex++;
			break;
			
		case 10:
			if( !LCD_ScrollText(lcd, stext, false) ) {
				waitTime = 200;
//				testIndex++;
				testIndex = 5;
			}
			waitTime = 100;
			break;
			
		default:
			if(--numberOfIterations < -999)
				numberOfIterations = 1000;
			waitTime = 0;
			testIndex = 0;
			break;
		}
		
		while(waitTime--) {
			Delay(1);
			
			// do stuff here
			ArrowsCheck(lcd);
			BatteryCheck(lcd);
		}

	}
	
	/* Segment bits */
	LCD_AllOff(lcd);
	LCD_Disable(lcd);
}

void ArrowsCheck(LCD_TypeDef *lcd)
{
#define BUTTON_BUZZER_TIME 20
	static int counter = BUTTON_BUZZER_TIME;
	
	GPIO_TypeDef *gpio = GPIO;

	uint32_t portVal = ~gpio->P[4].DIN; // get contents of port E
	
	LCD_Symbol(lcd, LCD_SYMBOL_ARR_L, portVal & 0x01);
	LCD_Symbol(lcd, LCD_SYMBOL_ARR_UP, portVal & 0x02);
	LCD_Symbol(lcd, LCD_SYMBOL_ARR_DN, portVal & 0x04);
	LCD_Symbol(lcd, LCD_SYMBOL_ARR_R, portVal & 0x08);
	
	if(portVal & 0x000F) {
		if(counter) {
			bzEnable = true;
			counter--;
		} else {
			bzEnable = false;
		}
	} else {
		bzEnable = false;
		counter = BUTTON_BUZZER_TIME;
	}
	
	if( (portVal & 0x000F) == 0x000F )
		KillDevice(); // GO TO EM4 HERE!!!!
}

void BuzzerCtrl(void)
{
	GPIO_TypeDef *gpio = GPIO;
	
	if(bzEnable) {
		if(gpio->P[0].DOUT & (1 << 12)) {
			gpio->P[0].DOUT &= ~(1 << 12);
			gpio->P[0].DOUT |= (1 << 13);
		} else {
			gpio->P[0].DOUT &= ~(1 << 13);
			gpio->P[0].DOUT |= (1 << 12);			
		}
	} else {
		gpio->P[0].DOUT &= ~( (1 << 12) | (1 << 13) );
	}
}

void BatteryCheck(LCD_TypeDef *lcd)
{
#define BAT_SAMPLE_PERIOD 100;
	
	static int counter = BAT_SAMPLE_PERIOD;
	static int battLevels[] = {30, 33, 36};
	static int battLevelIndex = 0;
	
	if(--counter)
		return;
	counter = BAT_SAMPLE_PERIOD;
	
	switch( VCMP_GetResult() ){
	case 0: // still busy
		break;
	case 1: // vdd above set
		if(battLevelIndex >= (sizeof(battLevels) / sizeof(battLevels[0]))) {
			LCD_Battery(lcd, 4);
			battLevelIndex = 0;
			VCMP_SetLevel(battLevels[battLevelIndex]);
		} else {
			VCMP_SetLevel(battLevels[++battLevelIndex]);
		}
		break;
		
	case -1: // vdd below set
		LCD_Battery(lcd, battLevelIndex + 1);
		battLevelIndex = 0;
		VCMP_SetLevel(battLevels[battLevelIndex]);
		break;
	}
}

/* call every half second */
void ClockUpdate(void)
{
	static unsigned int timeCount = 0;
	static unsigned long killTimeCount = 0;

	// kill device after a period of time
	if( ++killTimeCount > (10 * 7200) )
		KillDevice();
	
	if(++timeCount > (5999 * 2))
		timeCount = 0;
	
	LCD_Symbol(LCD, LCD_SYMBOL_10COL, timeCount & 0x01);
	
	int val;
	val = timeCount / 120;
	val *= 100;
	val += (timeCount % 120) / 2;
	
	LCD_Number(LCD, val);
}

void KillDevice(void)
{
	int i;

	/* Write "magic pattern" (see EFM32 reference manual) to enable EM4 */
	/* Sequence is 2,3,2,3,2,3,2,3 */
	for (i = 0; i < 4; i++)
	{
		EMU->CTRL = 2 << _EMU_CTRL_EM4CTRL_SHIFT;
		EMU->CTRL = 3 << _EMU_CTRL_EM4CTRL_SHIFT;
	}
	EMU->CTRL = 2 << 2;
	/* Reset device to get back.... */
	
	while(1)
		;
}